import base64, httpclient, json, sequtils, strutils, times, uri

type
  ChangeLog* = object
    id*: string
    historyId*: string
    issueId*: string
    author*: string
    createdAt*: DateTime
    field*: string
    oldValue*: string
    newValue*: string

  Issue* = object
    id*: string
    issueType*: string
    summary*: string
    epicId*: string
    assignee*: string
    status*: string
    priority*: string
    affectsVersions*: seq[string]
    fixVersions*: seq[string]
    resolution*: string
    testPhase*: string
    teams*: seq[string]

  LinkedIssue* = object
    id*: int
    toId*: string
    fromId*: string
    linkType*: string

let client = newHttpClient()
var API_BASE = "";
const FIELDS = "issuetype,summary,customfield_10014,assignee,status,priority,issuelinks,customfield_10218,changelog,resolution,versions,fixVersions,customfield_10001"

proc parseIssue(json: JsonNode): (Issue, seq[ChangeLog], seq[LinkedIssue]) =
  let f = json["fields"]

  let issue = Issue(
    id: json["key"].getStr(),
    issueType: f["issuetype"]["name"].getStr(),
    summary: f["summary"].getStr(),
    epicId: f["customfield_10014"].getStr(),
    assignee:
      if f["assignee"].kind == JNull: "Unassigned"
      else: f["assignee"]["displayName"].getStr(),
    status: f["status"]["name"].getStr(),
    priority:
      if f["priority"].kind == JObject: f["priority"]["name"].getStr()
      else: "",
    resolution:
      if f["resolution"].kind == JNull: ""
      else: f["resolution"]["name"].getStr(),
    affectsVersions:
      if f["versions"].getElems().len > 0:
        f["versions"].getElems().mapIt(it["name"].getStr())
      else: @[],
    fixVersions:
      if f["fixVersions"].getElems().len > 0:
        f["fixVersions"].getElems().mapIt(it["name"].getStr())
      else: @[],
    testPhase:
        if f["customfield_10218"].kind == JNull: ""
        else: f["customfield_10218"]["value"].getStr(),
    teams:
      if f["customfield_10001"].getElems().len > 0:
        f["customfield_10001"].getElems.mapIt(it[""].getStr())
      else : @[]
  )

  let changelogs =
    if json.hasKey("changelog") and json["changelog"]["histories"].getElems().len > 0:
      json["changelog"]["histories"].getElems().map(
        proc (h: JsonNode): seq[ChangeLog] = h["items"].mapIt(
          ChangeLog(
            historyId: h["id"].getStr(),
            issueId: json["key"].getStr(),
            author:
              if h.hasKey("author"): h["author"]["displayName"].getStr()
              else: "",
            createdAt: parse(
              h["created"].getStr()[0..17] & h["created"].getStr()[^6..^3],
              "yyyy-MM-dd'T'HH:mm:sszz"),
            field: it["field"].getStr(),
            oldValue: it["fromString"].getStr(),
            newValue: it["toString"].getStr()
          )
        )
      ).foldl(a & b)
    else: @[]

  let linkedIssues =
    if f.hasKey("issuelinks") and f["issuelinks"].getElems().len > 0:
      f["issuelinks"].mapIt(
        if it.hasKey("inwardIssue"):
          @[
            LinkedIssue(
              fromId: json["key"].getStr(),
              toId: it["inwardIssue"]["key"].getStr(),
              linkType: it["type"]["inward"].getStr()),
            LinkedIssue(
              toId: json["key"].getStr(),
              fromId: it["inwardIssue"]["key"].getStr(),
              linkType: it["type"]["outward"].getStr())
          ]
        else:
          @[
            LinkedIssue(
              fromId: json["key"].getStr(),
              toId: it["outwardIssue"]["key"].getStr(),
              linkType: it["type"]["outward"].getStr()),
            LinkedIssue(
              toId: json["key"].getStr(),
              fromId: it["outwardIssue"]["key"].getStr(),
              linkType: it["type"]["inward"].getStr())
          ]
      ).foldl(a & b)
    else: @[]

  result = (issue, changelogs, linkedIssues)

proc initJiraClient*(apiBasePath: string, username: string, apiToken: string) =
  API_BASE = apiBasePath
  client.headers = newHttpHeaders({
    "Content-Type": "application/json",
    "Authorization": "Basic " & encode(username & ":" & apiToken)
  })

proc searchIssues*(jql: string, includeChangelog: bool = false):
  (seq[Issue], seq[ChangeLog], seq[LinkedIssue]) =

  result = (@[], @[], @[])

  var query = @[
    ("jql", jql),
    ("fields", FIELDS)
  ]

  if includeChangelog: query.add(("expand", "changelog"))

  var resp = client.get(API_BASE & "/rest/api/3/search?" & encodeQuery(query))

  while true:
    if not resp.status.startsWith("2"):
      raise newException(Exception,
        "Received error from API: " & resp.status &
        "\nHeaders: " & $resp.headers &
        "\nBody: " & $resp.body)

    let body = parseJson(resp.body)
    let nextStartAt = body["startAt"].getInt(0) + body["maxResults"].getInt(0)

    echo "Retrieved records " &
      $body["startAt"].getInt() & " to " &
      $(nextStartAt - 1) & " of " &
      $body["total"].getInt() &
      " (" & $body["issues"].getElems().len & " records received)"

    let issuesLogsAndLinks = body["issues"].getElems().mapIt(parseIssue(it))

    if issuesLogsAndLinks.len > 0:
      result[0] &= issuesLogsAndLinks.mapIt(it[0])
      result[1] &= issuesLogsAndLinks.mapIt(it[1]).foldl(a & b)
      result[2] &= issuesLogsAndLinks.mapIt(it[2]).foldl(a & b)

    if nextStartAt > body["total"].getInt(): break

    resp = client.get(
      API_BASE & "/rest/api/3/search?" &
      encodeQuery(query & ("startAt", $nextStartAt)))
